@@ -1,7 +1,7 @@
 #
 #
 
-# Copyright (C) 2007, 2008, 2010 Google Inc.
+# Copyright (C) 2007, 2008, 2010, 2012 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -323,6 +323,15 @@ class HttpVersionNotSupported(HttpException):
   code = 505
 
 
+def ParseHeaders(buf):
+  """Parses HTTP headers.
+
+  @note: This is just a trivial wrapper around C{mimetools.Message}
+
+  """
+  return mimetools.Message(buf, 0)
+
+
 def SocketOperation(sock, op, arg1, timeout):
   """Wrapper around socket functions.
 
@@ -521,19 +530,6 @@ def Handshake(sock, write_timeout):
     raise HttpError("Error in SSL handshake: %s" % err)
 
 
-def InitSsl():
-  """Initializes the SSL infrastructure.
-
-  This function is idempotent.
-
-  """
-  if not OpenSSL.rand.status():
-    raise EnvironmentError("OpenSSL could not collect enough entropy"
-                           " for the PRNG")
-
-  # TODO: Maybe add some additional seeding for OpenSSL's PRNG
-
-
 class HttpSslParams(object):
   """Data class for SSL key and certificate.
 
@@ -998,7 +994,7 @@ class HttpMessageReader(object):
     """
     # Parse headers
     self.header_buffer.seek(0, 0)
-    self.msg.headers = mimetools.Message(self.header_buffer, 0)
+    self.msg.headers = ParseHeaders(self.header_buffer)
 
     self.peer_will_close = self._WillPeerCloseConnection()
 
