@@ -30,6 +30,7 @@ from ganeti.hypervisor import hv_fake
 from ganeti.hypervisor import hv_xen
 from ganeti.hypervisor import hv_kvm
 from ganeti.hypervisor import hv_chroot
+from ganeti.hypervisor import hv_lxc
 
 
 _HYPERVISOR_MAP = {
@@ -38,6 +39,7 @@ _HYPERVISOR_MAP = {
   constants.HT_FAKE: hv_fake.FakeHypervisor,
   constants.HT_KVM: hv_kvm.KVMHypervisor,
   constants.HT_CHROOT: hv_chroot.ChrootManager,
+  constants.HT_LXC: hv_lxc.LXCHypervisor,
   }
 
 
