@@ -48,6 +48,7 @@ _DISK_LETTERS = string.ascii_lowercase
 _FILE_DRIVER_MAP = {
   constants.FD_LOOP: "file",
   constants.FD_BLKTAP: "tap:aio",
+  constants.FD_BLKTAP2: "tap2:tapdisk:aio",
   }
 
 
@@ -584,8 +585,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
     """Start an instance.
 
     """
-    startup_memory = self._InstanceStartupMemory(instance,
-                                                 hvparams=instance.hvparams)
+    startup_memory = self._InstanceStartupMemory(instance)
 
     self._MakeConfigFile(instance, startup_memory, block_devices)
 
