@@ -36,6 +36,7 @@ from ganeti import constants
 from ganeti import errors
 from ganeti import utils
 from ganeti import netutils
+from ganeti import compat
 
 
 #: Used to recognize point at which socat(1) starts to listen on its socket.
@@ -56,7 +57,7 @@ SOCAT_LOG_WARNING = "W"
 SOCAT_LOG_ERROR = "E"
 SOCAT_LOG_FATAL = "F"
 
-SOCAT_LOG_IGNORE = frozenset([
+SOCAT_LOG_IGNORE = compat.UniqueFrozenset([
   SOCAT_LOG_DEBUG,
   SOCAT_LOG_INFO,
   SOCAT_LOG_NOTICE,
@@ -93,7 +94,8 @@ SOCAT_OPTION_MAXLEN = 400
  PROG_DD,
  PROG_DD_PID,
  PROG_EXP_SIZE) = range(1, 6)
-PROG_ALL = frozenset([
+
+PROG_ALL = compat.UniqueFrozenset([
   PROG_OTHER,
   PROG_SOCAT,
   PROG_DD,
@@ -216,7 +218,7 @@ class CommandBuilder(object):
       # second address is only used for writing
       "-u",
 
-      ",".join(addr1), ",".join(addr2)
+      ",".join(addr1), ",".join(addr2),
       ]
 
   def _GetMagicCommand(self):
@@ -262,9 +264,9 @@ class CommandBuilder(object):
       dd_cmd.write(" && ")
 
     dd_cmd.write("{ ")
-    # Setting LC_ALL since we want to parse the output and explicitely
-    # redirecting stdin, as the background process (dd) would have /dev/null as
-    # stdin otherwise
+    # Setting LC_ALL since we want to parse the output and explicitly
+    # redirecting stdin, as the background process (dd) would have
+    # /dev/null as stdin otherwise
     dd_cmd.write("LC_ALL=C dd bs=%s <&0 2>&%d & pid=${!};" %
                  (BUFSIZE, self._dd_stderr_fd))
     # Send PID to daemon
