@@ -180,6 +180,7 @@ def SetupCluster(rapi_user):
     qa_node.MarkNodeAddedAll()
 
   RunTestIf("test-jobqueue", qa_cluster.TestJobqueue)
+  RunTestIf("test-jobqueue", qa_job.TestJobCancellation)
 
   # enable the watcher (unconditionally)
   RunTest(qa_daemon.TestResumeWatcher)
@@ -385,6 +386,7 @@ def RunNetworkTests():
   """
   RunTestIf("network", qa_network.TestNetworkAddRemove)
   RunTestIf("network", qa_network.TestNetworkConnect)
+  RunTestIf(["network", "tags"], qa_network.TestNetworkTags)
 
 
 def RunGroupRwTests():
