@@ -34,6 +34,7 @@ module Ganeti.DataCollectors.Types
   , DCStatusCode(..)
   , DCVersion(..)
   , buildReport
+  , mergeStatuses
   ) where
 
 import Data.Char
@@ -116,6 +117,17 @@ addStatus dcStatus value = makeObj
   , ("data", value)
   ]
 
+-- | Helper function for merging statuses.
+mergeStatuses :: (DCStatusCode, String) -> (DCStatusCode, [String])
+              -> (DCStatusCode, [String])
+mergeStatuses (newStat, newStr) (storedStat, storedStrs) =
+  let resStat = max newStat storedStat
+      resStrs =
+        if newStr == ""
+          then storedStrs
+          else storedStrs ++ [newStr]
+  in (resStat, resStrs)
+
 -- | Utility function for building a report automatically adding the current
 -- timestamp (rounded up to seconds).
 -- If the version is not specified, it will be set to the value indicating
