@@ -33,8 +33,6 @@ import System.Log.Logger
 import Test.Ganeti.TestImports ()
 import Test.Ganeti.Attoparsec
 import Test.Ganeti.BasicTypes
-import Test.Ganeti.Block.Drbd.Parser
-import Test.Ganeti.Block.Drbd.Types
 import Test.Ganeti.Common
 import Test.Ganeti.Confd.Utils
 import Test.Ganeti.Confd.Types
@@ -66,6 +64,10 @@ import Test.Ganeti.Query.Query
 import Test.Ganeti.Rpc
 import Test.Ganeti.Runtime
 import Test.Ganeti.Ssconf
+import Test.Ganeti.Storage.Diskstats.Parser
+import Test.Ganeti.Storage.Drbd.Parser
+import Test.Ganeti.Storage.Drbd.Types
+import Test.Ganeti.Storage.Lvm.LVParser
 import Test.Ganeti.THH
 import Test.Ganeti.Types
 import Test.Ganeti.Utils
@@ -91,6 +93,7 @@ allTests =
   , testConfd_Types
   , testConfd_Utils
   , testDaemon
+  , testBlock_Diskstats_Parser
   , testBlock_Drbd_Parser
   , testBlock_Drbd_Types
   , testErrors
@@ -120,6 +123,7 @@ allTests =
   , testRpc
   , testRuntime
   , testSsconf
+  , testStorage_Lvm_LVParser
   , testTHH
   , testTypes
   , testUtils
