@@ -135,6 +135,20 @@ class TestQmpMessage(testutils.GanetiTestCase):
 
     rebuilt_message = hv_kvm.QmpMessage.BuildFromJsonString(serialized)
     self.assertEqual(rebuilt_message, message)
+    self.assertEqual(len(rebuilt_message), len(test_data))
+
+  def testDelete(self):
+    toDelete = "execute"
+    test_data = {
+      toDelete: "command",
+      "arguments": ["a", "b", "c"],
+      }
+    message = hv_kvm.QmpMessage(test_data)
+
+    oldLen = len(message)
+    del(message[toDelete])
+    newLen = len(message)
+    self.assertEqual(oldLen - 1, newLen)
 
 
 class TestQmp(testutils.GanetiTestCase):
@@ -185,35 +199,37 @@ class TestQmp(testutils.GanetiTestCase):
 
 
 class TestConsole(unittest.TestCase):
-  def _Test(self, instance, hvparams):
-    cons = hv_kvm.KVMHypervisor.GetInstanceConsole(instance, hvparams, {})
+  def _Test(self, instance, node, hvparams):
+    cons = hv_kvm.KVMHypervisor.GetInstanceConsole(instance, node, hvparams, {})
     self.assertTrue(cons.Validate())
     return cons
 
   def testSerial(self):
     instance = objects.Instance(name="kvm.example.com",
-                                primary_node="node6017")
+                                primary_node="node6017-uuid")
+    node = objects.Node(name="node6017", uuid="node6017-uuid")
     hvparams = {
       constants.HV_SERIAL_CONSOLE: True,
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, hvparams)
+    cons = self._Test(instance, node, hvparams)
     self.assertEqual(cons.kind, constants.CONS_SSH)
-    self.assertEqual(cons.host, instance.primary_node)
+    self.assertEqual(cons.host, node.name)
     self.assertEqual(cons.command[0], pathutils.KVM_CONSOLE_WRAPPER)
     self.assertEqual(cons.command[1], constants.SOCAT_PATH)
 
   def testVnc(self):
     instance = objects.Instance(name="kvm.example.com",
-                                primary_node="node7235",
+                                primary_node="node7235-uuid",
                                 network_port=constants.VNC_BASE_PORT + 10)
+    node = objects.Node(name="node7235", uuid="node7235-uuid")
     hvparams = {
       constants.HV_SERIAL_CONSOLE: False,
       constants.HV_VNC_BIND_ADDRESS: "192.0.2.1",
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, hvparams)
+    cons = self._Test(instance, node, hvparams)
     self.assertEqual(cons.kind, constants.CONS_VNC)
     self.assertEqual(cons.host, "192.0.2.1")
     self.assertEqual(cons.port, constants.VNC_BASE_PORT + 10)
@@ -223,12 +239,13 @@ class TestConsole(unittest.TestCase):
     instance = objects.Instance(name="kvm.example.com",
                                 primary_node="node7235",
                                 network_port=11000)
+    node = objects.Node(name="node7235", uuid="node7235-uuid")
     hvparams = {
       constants.HV_SERIAL_CONSOLE: False,
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: "192.0.2.1",
       }
-    cons = self._Test(instance, hvparams)
+    cons = self._Test(instance, node, hvparams)
     self.assertEqual(cons.kind, constants.CONS_SPICE)
     self.assertEqual(cons.host, "192.0.2.1")
     self.assertEqual(cons.port, 11000)
@@ -237,12 +254,13 @@ class TestConsole(unittest.TestCase):
     instance = objects.Instance(name="kvm.example.com",
                                 primary_node="node24325",
                                 network_port=0)
+    node = objects.Node(name="node24325", uuid="node24325-uuid")
     hvparams = {
       constants.HV_SERIAL_CONSOLE: False,
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, hvparams)
+    cons = self._Test(instance, node, hvparams)
     self.assertEqual(cons.kind, constants.CONS_MESSAGE)
 
 
