@@ -34,11 +34,13 @@ import testutils
 
 class TestConsole(unittest.TestCase):
   def test(self):
-    instance = objects.Instance(name="lxc.example.com", primary_node="node199")
-    cons = hv_lxc.LXCHypervisor.GetInstanceConsole(instance, {}, {})
+    instance = objects.Instance(name="lxc.example.com",
+                                primary_node="node199-uuid")
+    node = objects.Node(name="node199", uuid="node199-uuid")
+    cons = hv_lxc.LXCHypervisor.GetInstanceConsole(instance, node, {}, {})
     self.assertTrue(cons.Validate())
     self.assertEqual(cons.kind, constants.CONS_SSH)
-    self.assertEqual(cons.host, instance.primary_node)
+    self.assertEqual(cons.host, node.name)
     self.assertEqual(cons.command[-1], instance.name)
 
 
